@@ -142,7 +142,7 @@ class NaturalDeductionSystem:
         self.language = language
         self.rules = rules
 
-    def is_correct_derivation(self, derivation, inference=None, return_error_list=False):
+    def is_correct_derivation(self, derivation, inference=None, return_error_list=False, exit_on_first_error=False):
         """Determines if a derivation has been correctly performed.
 
         Will check that the steps with a justification of 'premise' are premises of inference, that every other step
@@ -160,6 +160,8 @@ class NaturalDeductionSystem:
         return_error_list: bool, optional
             If False, will just return True or False (exits when it finds an error, more efficient) If True, will return
             a tuple (boolean, [error_list]) (computes all errors, does not exit on the first, less efficient)
+        exit_on_first_error: bool, optional
+            If `return_error_list` and this are both true, it will return a list with a single error instead of many.
 
         Examples
         --------
@@ -202,7 +204,7 @@ class NaturalDeductionSystem:
         >>> correct
         False
         >>> error_list
-        ['Step 3: Incorrect handling of suppositions in on_step 0, it is in a closed supposition']
+        [(3, 'Incorrect handling of suppositions in on step 0, it is in a closed supposition')]
 
         Incorrectly closing a supposition, with a rule that does not allow that
 
@@ -216,7 +218,21 @@ class NaturalDeductionSystem:
         >>> correct
         False
         >>> error_list
-        ['Step 3: Incorrect handling of suppositions']
+        [(3, 'Incorrect handling of suppositions')]
+
+        Notes
+        --------
+        For rules that have more than one variant (e.g. ``E∧1`` and ``E∧2``) you can omit the number. E.g.:
+
+        >>> from logics.utils.parsers import classical_parser
+        >>> from logics.instances.propositional.natural_deduction import classical_natural_deduction_system
+        >>> inf = classical_parser.parse('p and q / p')
+        >>> deriv = classical_parser.parse_derivation('''
+        ... p and q; premise; []; []
+        ... p; E∧; [0]; []''',
+        ... natural_deduction=True)
+        >>> classical_natural_deduction_system.is_correct_derivation(deriv, inf)
+        True
         """
         error_list = list()
 
@@ -229,22 +245,28 @@ class NaturalDeductionSystem:
                     if not return_error_list:
                         return False
                     else:
-                        error_list.append(f"Step {step_index}: {step.content} was marked as 'premise', "
-                                          f"but is not a premise of the inference given")
+                        error_list.append((step_index,
+                                           "was marked as 'premise', but is not a premise of the inference given"))
+                        if exit_on_first_error:
+                            return False, error_list
                 # premise steps need to have the same supposition level than the previous step
                 if step_index == 0:
                     if step.open_suppositions:
                         if not return_error_list:
                             return False
                         else:
-                            error_list.append(f"Step 0: Incorrect supposition handling. "
-                                              f"Premise steps do not open suppositions")
+                            error_list.append((0, "Incorrect supposition handling. "
+                                                  "Premise steps do not open suppositions"))
+                            if exit_on_first_error:
+                                return False, error_list
                 else:
                     if step.open_suppositions != derivation[step_index-1].open_suppositions:
                         if not return_error_list:
                             return False
                         else:
-                            error_list.append(f"Step {step_index}: Incorrect supposition handling")
+                            error_list.append((step_index, "Incorrect supposition handling"))
+                            if exit_on_first_error:
+                                return False, error_list
 
             # The justification is 'supposition'
             elif step.justification == 'supposition':
@@ -253,38 +275,76 @@ class NaturalDeductionSystem:
                         if not return_error_list:
                             return False
                         else:
-                            error_list.append("Step 0: Incorrect supposition handling")
+                            error_list.append((0, "Incorrect supposition handling"))
+                            if exit_on_first_error:
+                                return False, error_list
                 else:
                     if not (step.open_suppositions[:-1] == derivation[step_index - 1].open_suppositions and
                             step.open_suppositions[-1] == step_index):
                         if not return_error_list:
                             return False
                         else:
-                            error_list.append(f"Step {step_index}: Incorrect supposition handling")
+                            error_list.append((step_index, "Incorrect supposition handling"))
+                            if exit_on_first_error:
+                                return False, error_list
 
             # If the justification is the name of a specific rule
             else:
-                if step.justification not in self.rules:
+                # Second conjunct is for rules that have two or more versions
+                if step.justification not in self.rules and f"{step.justification}1" not in self.rules:
                     if not return_error_list:
                         return False
                     else:
-                        error_list.append(f"Step {step_index}: Justification is incorrect, must be either "
-                                          f"'premise', 'supposition', or the name of a specific axiom or rule")
+                        error_list.append((step_index, "Justification is incorrect, must be either "
+                                          "'premise', 'supposition', or the name of a specific axiom or rule"))
+                        if exit_on_first_error:
+                            return False, error_list
+                else:  # Only check correct application of the rule if valid rule
+                    # Get the rule names we need to try (might be something like ['I∧'] or like ['E∧1', 'E∧2'])
+                    if step.justification in self.rules:
+                        rule_names_to_try = [step.justification]
+                    else:
+                        counter = 1
+                        exit_loop = False
+                        rule_names_to_try = list()
+                        while not exit_loop:
+                            if f"{step.justification}{counter}" in self.rules:
+                                rule_names_to_try.append(f"{step.justification}{counter}")
+                                counter += 1
+                            else:
+                                exit_loop = True
 
-                correct, error = self.is_correct_application(derivation=derivation[:step_index+1], step=-1,  # last step
-                                                             rule=self.rules[step.justification], return_error=True)
-                if not correct:
-                    if not return_error_list:
-                        return False
-                    else:
-                        error_list.append(f'Step {step_index}: {error}')
+                    # Try to apply each of the relevant rule names
+                    correct = False
+                    for rule_name in rule_names_to_try:
+                        correct, error = self.is_correct_application(derivation=derivation[:step_index+1],
+                                                                     step=-1,  # last step
+                                                                     rule=self.rules[rule_name],
+                                                                     return_error=True)
+                        if correct:
+                            break  # if one of the rules to try is correct, exit
+
+                    if not correct:
+                        if not return_error_list:
+                            return False
+                        else:
+                            if len(rule_names_to_try) == 1:
+                                error_list.append((step_index, error))
+                            else:
+                                # If there is more than one possible rule, return a generic error message
+                                error_list.append((step_index, "Incorrect application of rule"))
+                            if exit_on_first_error:
+                                    return False, error_list
 
         # Finally, checks if the last step is the conclusion of the inference
         if inference is not None and derivation.conclusion != inference.conclusion:
             if not return_error_list:
                 return False
             else:
-                error_list.append(f"Final step of the derivation is not the conclusion of the inference given")
+                error_list.append((len(derivation)-1,
+                                   "Final step of the derivation is not the conclusion of the inference given"))
+                if exit_on_first_error:
+                    return False, error_list
 
         # If it gets to here either there are no errors, or there are some but return_error_list is True
         if not error_list:
@@ -330,7 +390,7 @@ class NaturalDeductionSystem:
         False
         >>> classical_natural_deduction_system.is_correct_application(deriv, 3, classical_natural_deduction_system.rules['E→'],
         ...                                                           return_error=True)
-        (False, 'Incorrect handling of suppositions in on_step 0, it is in a closed supposition')
+        (False, 'Incorrect handling of suppositions in on step 0, it is in a closed supposition')
         """
         last_step = derivation[step]
 
@@ -348,12 +408,17 @@ class NaturalDeductionSystem:
 
         # Check that the conclusion and the premises of the derivation are instances of the rule
         # Conclusion
-        if rule[-1].justification is not None and last_step.justification != rule[-1].justification:
-            if not return_error:
-                return False
-            return False, f"Justification for step {derivation.index(last_step)} does not coincide with " \
-                          f"the justification in the rule conclusion"
-
+        # (justification)
+        if rule[-1].justification is not None:
+            exact_match = last_step.justification == rule[-1].justification
+            without_num_match = rule[-1].justification[-1].isdigit() and \
+                                rule[-1].justification[:-1] == last_step.justification
+            if not exact_match and not without_num_match:
+                if not return_error:
+                    return False
+                return False, f"Justification for step {derivation.index(last_step)} does not coincide with " \
+                              f"the justification in the rule conclusion"
+        # (content)
         instance, subst_dict = last_step.content.is_instance_of(rule[-1].content, self.language, return_subst_dict=True)
         if not instance:
             if not return_error:
@@ -426,13 +491,13 @@ class NaturalDeductionSystem:
                             relevant_sup_dict[relevant_step] not in derivation[step_number].open_suppositions:
                         if not return_error:
                             return False
-                        return False, f"Incorrect handling of suppositions in on_step {step_number}"
+                        return False, f"Incorrect handling of suppositions in on step {step_number}"
                     # Check that closed suppositions of the rule are not open in the derivation
                     if relevant_step not in rule_step.open_suppositions and \
                             relevant_sup_dict[relevant_step] in derivation[step_number].open_suppositions:
                         if not return_error:
                             return False
-                        return False, f"Incorrect handling of suppositions in on_step {step_number}"
+                        return False, f"Incorrect handling of suppositions in on step {step_number}"
 
                 # Outside the rule, no steps in closed suppositions are being used
                 # (if the on_step has an open supposition, it must also be present in the conclusion, otherwise we are
@@ -442,14 +507,14 @@ class NaturalDeductionSystem:
                         if open_sup not in last_step.open_suppositions:
                             if not return_error:
                                 return False
-                            return False, f"Incorrect handling of suppositions in on_step {step_number}, it " \
+                            return False, f"Incorrect handling of suppositions in on step {step_number}, it " \
                                           f"is in a closed supposition"
 
         # Conclusion again (check that it immediately follows the last step, if it corresponds)
         if prev_step is not None and derivation.index(last_step) != prev_step + 1:
             if not return_error:
                 return False
-            return False, f"On step {derivation.index(last_step)} does not immediately follow the previous on_step"
+            return False, f"On step {derivation.index(last_step)} does not immediately follow the previous on step"
 
         # Supposition checking in the conclusion
         for relevant_step in relevant_sup_dict:
